<?php
namespace DsbNext\Services;

class DashboardTAControllerService {
    
    public function __construct($entityManager, $environment){
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    /**
     * These 3 functions perform the drilldown of the AFGeneral chart.
     * 
     * They receive parameters, which are passed in the function of the same name in DashboardTaController.
     * 
     * A query is made with the indicated parameters, which returns the values to the DashboardTaController 
     * where they are added to the dataset.
     * 
     * @access public
     * @author Arthur Gustavo
     */
    
    public function drillDownAFGeneralFirstLevel($params) {
        $query = "
            SELECT 
                    NMOPERADOR AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE,
                    NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO, 
                    CDOPERADOR
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                       AND NVL(IDCONFRECEBIMENTO, 'X') = :IDCONFRECEBIMENTO
                )
                GROUP BY 
                    IDCONFRECEBIMENTO,
                    CDOPERADOR,
                    NMOPERADOR
                ORDER BY 
                    CDOPERADOR
        ";
        try{
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            return $result;
        }catch(Exception $e) {
          throw $e; 
        }
    }
    
    public function drillDownAFGeneralSecondLevel($params) {
        $query = "
            SELECT
                NMREGIAOGEO AS LABEL,
                COUNT(NRAUTOFORN) AS VALUE,
                NVL(IDCONFRECEBIMENTO,'X') AS IDCONFRECEBIMENTO,
                CDOPERADOR,
                NMREGIAOGEO AS REGIAO
            FROM
            (
                SELECT * 
                  FROM DM_TRACKING_AF
                 WHERE CDFILCENTCMP = :CDFILCENTCMP
                   AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                   AND CDOPERADOR = :CDOPERADOR
                   AND NVL(IDCONFRECEBIMENTO, 'X') = :IDCONFRECEBIMENTO
            )
            GROUP BY
                NMREGIAOGEO,
                CDOPERADOR,
                IDCONFRECEBIMENTO
            ORDER BY
                NMREGIAOGEO
        ";
        try{
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            return $result;
        }catch(Exception $e) {
          throw $e; 
        }
    }
    
    public function drillDownAFGeneralThirdLevel($params) {
        $query = "
                SELECT
                    NMESTADO AS LABEL,
                    COUNT(NRAUTOFORN) AS VALUE,
                    NVL(IDCONFRECEBIMENTO,'X') AS IDCONFRECEBIMENTO,
                    CDOPERADOR,
                    NMREGIAOGEO AS REGIAO,
                    NMESTADO
                FROM
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                       AND CDOPERADOR = :CDOPERADOR
                       AND NVL(IDCONFRECEBIMENTO,'X') = :IDCONFRECEBIMENTO
                       AND NMREGIAOGEO = :NMREGIAOGEO
                )
                GROUP BY
                    NMESTADO,
                    NMREGIAOGEO,
                    CDOPERADOR,
                    IDCONFRECEBIMENTO
                ORDER BY
                    NMESTADO
            ";
        try{
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            return $result;
        }catch(Exception $e) {
          throw $e; 
        }
    }
    
    public function drillDownAFBuyerFirstLevel($params) {
        $query = "SELECT 
                    CASE
                        IDCONFRECEBIMENTO
                        WHEN 'T' THEN 'Total'
                        WHEN 'P' THEN 'Parcial'
                        WHEN 'N' THEN 'Nâo Entregue'
                        WHEN 'D' THEN 'Devolucao'
                        ELSE 'Pendente'
                    END AS LABEL,
                    COUNT(CDFILDEST) AS VALUE,
                    CDOPERADOR,
                    NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO
                FROM 
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = :CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(FIRST_DAY(SYSDATE)) AND TRUNC(SYSDATE - 1)
                       AND CDOPERADOR = :CDOPERADOR
                )
                GROUP BY 
                    IDCONFRECEBIMENTO,
                    CDOPERADOR
                ORDER BY
                    DECODE(IDCONFRECEBIMENTO, 'T', 1, 'P', 2, 'N', 3, 'D', 4, 5)
            ";
        try{
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            return $result;
        }catch(Exception $e) {
          throw $e; 
        }
    }
    
    public function drillDownAFDeliveryLastYearFirstLevel($params) {
        $query = "
                SELECT
                    NMOPERADOR AS LABEL,
                    (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE)) AS TOTAL_AF,
                    ROUND(((SUM(TOTAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS TOTAL,
                    ROUND(((SUM(PARCIAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PARCIAL,
                    ROUND(((SUM(NAOENTREGUE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS NAOENTREGUE,
                    ROUND(((SUM(DEVOLUCAO) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS DEVOLUCAO,
                    ROUND(((SUM(PENDENTE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PENDENTE,
                    STARTDATE AS STARTDATE,
                    ENDDATE AS ENDDATE,
                    CDOPERADOR
                FROM
                (
                    SELECT * 
                      FROM DM_TRACKING_AF
                     WHERE CDFILCENTCMP = CDFILCENTCMP
                       AND DTPROGENTAF BETWEEN TRUNC(TO_DATE(STARTDATE, 'DD/MM/YYYY')) AND TRUNC(TO_DATE(ENDDATE, 'DD/MM/YYYY'))
                )
                PIVOT 
                (
                    COUNT ( NRAUTOFORN )
                    FOR IDCONFRECEBIMENTO
                    IN ( 'D' DEVOLUCAO,'T' TOTAL,'N' NAOENTREGUE,'P' PARCIAL,'X' PENDENTE )
                )
                GROUP BY
                    NMOPERADOR,
                    CDOPERADOR
                ORDER BY
                    CDOPERADOR
            ";
        try{
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            return $result;
        }catch(Exception $e) {
          throw $e; 
        }
    }
    
    public function drillDownAFDeliveryLastYearSecondLevel($params) {
        $query = "
                SELECT
                    NMREGIAOGEO AS LABEL, 
                    (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE)) AS TOTAL_AF,
                    ROUND(((SUM(TOTAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS TOTAL,
                    ROUND(((SUM(PARCIAL) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PARCIAL,
                    ROUND(((SUM(NAOENTREGUE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS NAOENTREGUE,
                    ROUND(((SUM(DEVOLUCAO) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS DEVOLUCAO,
                    ROUND(((SUM(PENDENTE) / (SUM(DEVOLUCAO) + SUM(TOTAL) + SUM(NAOENTREGUE) + SUM(PARCIAL) + SUM(PENDENTE))) * 100), 2) AS PENDENTE,
                    STARTDATE AS STARTDATE,
                    ENDDATE AS ENDDATE,
                    CDOPERADOR,
                    NMREGIAOGEO
                FROM 
                (
                    SELECT  NMOPERADOR,
                            NRAUTOFORN,
                            NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO, 
                            CDOPERADOR,
                            NMESTADO,
                            NMREGIAOGEO
                       FROM DM_TRACKING_AF
                      WHERE CDFILCENTCMP = CDFILCENTCMP
                        AND DTPROGENTAF BETWEEN TRUNC(TO_DATE(STARTDATE, 'DD/MM/YYYY')) AND TRUNC(TO_DATE(ENDDATE, 'DD/MM/YYYY'))
                        AND CDOPERADOR = CDOPERADOR
                )
                PIVOT 
                (
                    COUNT(NRAUTOFORN)
                    FOR IDCONFRECEBIMENTO IN ('D' DEVOLUCAO, 'T' TOTAL, 'N' NAOENTREGUE, 'P' PARCIAL, 'X' PENDENTE)
                )
                GROUP BY 
                    NMREGIAOGEO,
                    CDOPERADOR
                ORDER BY 
                    NMREGIAOGEO
            ";
        try{
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            return $result;
        }catch(Exception $e) {
          throw $e; 
        }
    }
    
    public function drillDownAFDeliveryLastYearThirdLevel($params) {
        $query = "
                SELECT 
                    NMESTADO AS LABEL,
                    TOTAL_AF,
                    ROUND((((ENTREGA_TOTAL * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as TOTAL,
                    ROUND((((ENTREGA_PARCIAL * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as PARCIAL,
                    ROUND((((NAO_ENTREGUE * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as NAOENTREGUE,
                    ROUND((((DEVOLUCAO * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as DEVOLUCAO,
                    ROUND((((PENDENTE * 100)/NVL(DECODE(TOTAL_AF, 0, 1,TOTAL_AF ), 1))),2) as PENDENTE,
                    STARTDATE AS STARTDATE,
                    ENDDATE AS ENDDATE,
                    CDOPERADOR,
                    NMREGIAOGEO,
                    NMESTADO
                FROM 
                (
                    SELECT 
                        NMESTADO, 
                        CDOPERADOR,
                        NMREGIAOGEO,
                        (SUM(DEVOLUCAO)+SUM(TOTAL)+ SUM(NAOENTREGUE)+ SUM(PARCIAL) + SUM(PENDENTE)) AS TOTAL_AF,
                        SUM(DEVOLUCAO) AS DEVOLUCAO, 
                        SUM(TOTAL) AS ENTREGA_TOTAL, 
                        SUM(NAOENTREGUE) AS NAO_ENTREGUE, 
                        SUM(PARCIAL) AS ENTREGA_PARCIAL, 
                        SUM(PENDENTE) AS PENDENTE
                    FROM 
                    (
                        SELECT NMOPERADOR,
                               NRAUTOFORN, 
                               NVL(IDCONFRECEBIMENTO, 'X') AS IDCONFRECEBIMENTO, 
                               CDOPERADOR, 
                               NMESTADO,
                               NMREGIAOGEO
                          FROM DM_TRACKING_AF
                         WHERE CDFILCENTCMP = CDFILCENTCMP
                           AND DTPROGENTAF BETWEEN TRUNC(TO_DATE(STARTDATE, 'DD/MM/YYYY')) AND TRUNC(TO_DATE(ENDDATE, 'DD/MM/YYYY'))
                           AND CDOPERADOR = CDOPERADOR
                           AND NMREGIAOGEO = NMREGIAOGEO
                    )
                    PIVOT 
                    (
                        COUNT(NRAUTOFORN)
                        FOR IDCONFRECEBIMENTO IN ('D' DEVOLUCAO, 'T' TOTAL, 'N' NAOENTREGUE, 'P' PARCIAL, 'X' PENDENTE)
                    )
                    GROUP BY 
                        NMESTADO,
                        CDOPERADOR,
                        NMREGIAOGEO
                    ORDER BY 
                        NMESTADO
                )
            ";
        try{
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $result = $stmt->fetchAll();
            return $result;
        }catch(Exception $e) {
          throw $e; 
        }
    }
}